# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from __future__ import annotations
from enum import Enum, auto
from typing import Dict, Optional, Tuple, Type


class WebSocketMessageParserError(Enum):
    FieldConversionFailed = auto()
    FieldFormatInvalid = auto()
    FieldTypeNotSupported = auto()
    TypeFormatInvalid = auto()
    TypeNotSupported = auto()

    def __str__(self):
        return str(self.name)


class WebSocketMessageParserException(Exception):
    def __init__(self, error: WebSocketMessageParserError):
        self.error = error
        super().__init__()


class WebSocketMessage:
    @classmethod
    def create_from_wsm(cls, wsm: str, is_strict: bool = False):
        raise NotImplementedError


def parse_wsm_header(
    msg: str,
) -> Tuple[Optional[str], Optional[WebSocketMessageParserError]]:
    lines = msg.splitlines(keepends=False)
    if len(lines) < 2:
        return None, WebSocketMessageParserError.TypeFormatInvalid

    # type
    type_line = lines[0]
    if not type_line.startswith("type: "):
        return None, WebSocketMessageParserError.TypeFormatInvalid
    type_ = type_line.removeprefix("type: ")

    return type_, None


def parse_wsm(
    wsm: str, models: Dict[str, Type[WebSocketMessage]], is_strict: bool = False
) -> Tuple[Optional[WebSocketMessage], Optional[WebSocketMessageParserError]]:
    type_, error = parse_wsm_header(wsm)
    if error:
        return None, error
    model = models.get(type_)
    if model is None:
        return None, WebSocketMessageParserError.TypeNotSupported
    try:
        return model.create_from_wsm(wsm, is_strict=is_strict), None
    except WebSocketMessageParserException as exception:
        return None, exception.error



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Ugc Service."""

__version__ = "2.5.4"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# admin_channel
from .wrappers import admin_create_channel
from .wrappers import admin_create_channel_async
from .wrappers import admin_delete_channel
from .wrappers import admin_delete_channel_async
from .wrappers import admin_get_channel
from .wrappers import admin_get_channel_async
from .wrappers import admin_update_channel
from .wrappers import admin_update_channel_async
from .wrappers import single_admin_delete_channel
from .wrappers import single_admin_delete_channel_async
from .wrappers import single_admin_get_channel
from .wrappers import single_admin_get_channel_async
from .wrappers import single_admin_update_channel
from .wrappers import single_admin_update_channel_async

# admin_content
from .wrappers import admin_delete_content
from .wrappers import admin_delete_content_async
from .wrappers import admin_delete_content_screenshot
from .wrappers import admin_delete_content_screenshot_async
from .wrappers import admin_download_content_preview
from .wrappers import admin_download_content_preview_async
from .wrappers import admin_get_content
from .wrappers import admin_get_content_async
from .wrappers import admin_get_specific_content
from .wrappers import admin_get_specific_content_async
from .wrappers import admin_get_user_content_by_share_code
from .wrappers import admin_get_user_content_by_share_code_async
from .wrappers import admin_hide_user_content
from .wrappers import admin_hide_user_content_async
from .wrappers import admin_search_channel_specific_content
from .wrappers import admin_search_channel_specific_content_async
from .wrappers import admin_search_content
from .wrappers import admin_search_content_async
from .wrappers import admin_update_content_direct
from .wrappers import admin_update_content_direct_async
from .wrappers import admin_update_content_s3
from .wrappers import admin_update_content_s3_async
from .wrappers import admin_update_screenshots
from .wrappers import admin_update_screenshots_async
from .wrappers import admin_upload_content_direct
from .wrappers import admin_upload_content_direct_async
from .wrappers import admin_upload_content_s3
from .wrappers import admin_upload_content_s3_async
from .wrappers import admin_upload_content_screenshot
from .wrappers import admin_upload_content_screenshot_async
from .wrappers import single_admin_delete_content
from .wrappers import single_admin_delete_content_async
from .wrappers import single_admin_get_content
from .wrappers import single_admin_get_content_async
from .wrappers import single_admin_update_content_direct
from .wrappers import single_admin_update_content_direct_async
from .wrappers import single_admin_update_content_s3
from .wrappers import single_admin_update_content_s3_async

# admin_group
from .wrappers import admin_create_group
from .wrappers import admin_create_group_async
from .wrappers import admin_delete_group
from .wrappers import admin_delete_group_async
from .wrappers import admin_get_all_groups
from .wrappers import admin_get_all_groups_async
from .wrappers import admin_get_group
from .wrappers import admin_get_group_async
from .wrappers import admin_get_group_contents
from .wrappers import admin_get_group_contents_async
from .wrappers import admin_update_group
from .wrappers import admin_update_group_async
from .wrappers import single_admin_delete_group
from .wrappers import single_admin_delete_group_async
from .wrappers import single_admin_get_all_groups
from .wrappers import single_admin_get_all_groups_async
from .wrappers import single_admin_get_group
from .wrappers import single_admin_get_group_async
from .wrappers import single_admin_get_group_contents
from .wrappers import single_admin_get_group_contents_async
from .wrappers import single_admin_update_group
from .wrappers import single_admin_update_group_async

# admin_tag
from .wrappers import admin_create_tag
from .wrappers import admin_create_tag_async
from .wrappers import admin_delete_tag
from .wrappers import admin_delete_tag_async
from .wrappers import admin_get_tag
from .wrappers import admin_get_tag_async
from .wrappers import admin_update_tag
from .wrappers import admin_update_tag_async

# admin_type
from .wrappers import admin_create_type
from .wrappers import admin_create_type_async
from .wrappers import admin_delete_type
from .wrappers import admin_delete_type_async
from .wrappers import admin_get_type
from .wrappers import admin_get_type_async
from .wrappers import admin_update_type
from .wrappers import admin_update_type_async

# anonymization
from .wrappers import admin_delete_all_user_channels
from .wrappers import admin_delete_all_user_channels_async
from .wrappers import admin_delete_all_user_contents
from .wrappers import admin_delete_all_user_contents_async
from .wrappers import admin_delete_all_user_group
from .wrappers import admin_delete_all_user_group_async
from .wrappers import admin_delete_all_user_states
from .wrappers import admin_delete_all_user_states_async
from .wrappers import delete_all_user_channel
from .wrappers import delete_all_user_channel_async
from .wrappers import delete_all_user_contents
from .wrappers import delete_all_user_contents_async
from .wrappers import delete_all_user_group
from .wrappers import delete_all_user_group_async
from .wrappers import delete_all_user_states
from .wrappers import delete_all_user_states_async

# public_channel
from .wrappers import create_channel
from .wrappers import create_channel_async
from .wrappers import delete_channel
from .wrappers import delete_channel_async
from .wrappers import get_channels
from .wrappers import get_channels_async
from .wrappers import update_channel
from .wrappers import update_channel_async

# public_content
from .wrappers import create_content_direct
from .wrappers import create_content_direct_async
from .wrappers import create_content_s3
from .wrappers import create_content_s3_async
from .wrappers import delete_content
from .wrappers import delete_content_async
from .wrappers import delete_content_screenshot
from .wrappers import delete_content_screenshot_async
from .wrappers import public_download_content_by_content_id
from .wrappers import public_download_content_by_content_id_async
from .wrappers import public_download_content_by_share_code
from .wrappers import public_download_content_by_share_code_async
from .wrappers import public_download_content_preview
from .wrappers import public_download_content_preview_async
from .wrappers import public_get_content_bulk
from .wrappers import public_get_content_bulk_async
from .wrappers import public_get_user_content
from .wrappers import public_get_user_content_async
from .wrappers import public_search_content
from .wrappers import public_search_content_async
from .wrappers import search_channel_specific_content
from .wrappers import search_channel_specific_content_async
from .wrappers import update_content_direct
from .wrappers import update_content_direct_async
from .wrappers import update_content_s3
from .wrappers import update_content_s3_async
from .wrappers import update_screenshots
from .wrappers import update_screenshots_async
from .wrappers import upload_content_screenshot
from .wrappers import upload_content_screenshot_async

# public_creator
from .wrappers import public_get_creator
from .wrappers import public_get_creator_async
from .wrappers import public_search_creator
from .wrappers import public_search_creator_async

# public_download_count
from .wrappers import add_download_count
from .wrappers import add_download_count_async

# public_follow
from .wrappers import get_followed_content
from .wrappers import get_followed_content_async
from .wrappers import get_followed_users
from .wrappers import get_followed_users_async
from .wrappers import get_public_followers
from .wrappers import get_public_followers_async
from .wrappers import get_public_following
from .wrappers import get_public_following_async
from .wrappers import update_user_follow_status
from .wrappers import update_user_follow_status_async

# public_group
from .wrappers import create_group
from .wrappers import create_group_async
from .wrappers import delete_group
from .wrappers import delete_group_async
from .wrappers import get_group
from .wrappers import get_group_async
from .wrappers import get_group_content
from .wrappers import get_group_content_async
from .wrappers import get_groups
from .wrappers import get_groups_async
from .wrappers import update_group
from .wrappers import update_group_async

# public_like
from .wrappers import get_liked_content
from .wrappers import get_liked_content_async
from .wrappers import update_content_like_status
from .wrappers import update_content_like_status_async

# public_tag
from .wrappers import get_tag
from .wrappers import get_tag_async

# public_type
from .wrappers import get_type
from .wrappers import get_type_async



import re

from typing import Any, Dict, List, Optional, Tuple


def validate_field(
    obj: Any,
    name: str,
    alias_map: Optional[Dict[str, str]] = None,
    required_map: Optional[Dict[str, bool]] = None,
    enum_map: Optional[Dict[str, List[Any]]] = None,
    pattern_map: Optional[Dict[str, re.Pattern]] = None,
    **kwargs,
) -> Tuple[bool, Optional[str]]:
    if alias_map is not None:
        alias = alias_map.get(name)
        if not alias:
            return False, f"Unrecognized field '{name}'."
    else:
        alias = name

    value = getattr(obj, alias) if hasattr(obj, alias) else None

    if required_map is not None:
        is_required = required_map.get(name, False)
        if is_required and value is None:
            return False, f"Required value '{alias}' was not set."

    if value is not None:
        is_value_list = isinstance(value, list)

        if enum_map is not None:
            enum_values = enum_map.get(name, [])
            if enum_values:
                if is_value_list:
                    is_valid = all(x in enum_values for x in value)
                    if not is_valid:
                        return (
                            False,
                            f"Some values in '{value}' are not part of the enum set '{enum_values}'.",
                        )
                else:
                    is_valid = value in enum_values
                    if not is_valid:
                        return (
                            False,
                            f"Value '{value}' is not part of the enum set '{enum_values}'.",
                        )

        if pattern_map is not None:
            pattern = pattern_map.get(name, None)
            if pattern:
                if is_value_list:
                    is_valid = all(pattern.match(x) for x in value)
                    if not is_valid:
                        return (
                            False,
                            f"Some values in '{value}' do not match the pattern '{pattern.pattern}'.",
                        )
                else:
                    is_valid = pattern.match(value)
                    if not is_valid:
                        return (
                            False,
                            f"Value '{value}' do not match the pattern '{pattern.pattern}'.",
                        )

    return True, None



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Seasonpass Service (1.15.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class StatusEnum(StrEnum):
    DRAFT = "DRAFT"
    PUBLISHED = "PUBLISHED"
    RETIRED = "RETIRED"


class SeasonSummary(Model):
    """A DTO object for season summary. (SeasonSummary)

    Properties:
        end: (end) REQUIRED str

        id_: (id) REQUIRED str

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        start: (start) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        pass_codes: (passCodes) OPTIONAL List[str]

        previous: (previous) OPTIONAL SeasonSummary

        published_at: (publishedAt) OPTIONAL str
    """

    # region fields

    end: str  # REQUIRED
    id_: str  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    start: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    pass_codes: List[str]  # OPTIONAL
    previous: SeasonSummary  # OPTIONAL
    published_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_end(self, value: str) -> SeasonSummary:
        self.end = value
        return self

    def with_id(self, value: str) -> SeasonSummary:
        self.id_ = value
        return self

    def with_name(self, value: str) -> SeasonSummary:
        self.name = value
        return self

    def with_namespace(self, value: str) -> SeasonSummary:
        self.namespace = value
        return self

    def with_start(self, value: str) -> SeasonSummary:
        self.start = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> SeasonSummary:
        self.status = value
        return self

    def with_pass_codes(self, value: List[str]) -> SeasonSummary:
        self.pass_codes = value
        return self

    def with_previous(self, value: SeasonSummary) -> SeasonSummary:
        self.previous = value
        return self

    def with_published_at(self, value: str) -> SeasonSummary:
        self.published_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "end"):
            result["end"] = str(self.end)
        elif include_empty:
            result["end"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "start"):
            result["start"] = str(self.start)
        elif include_empty:
            result["start"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "pass_codes"):
            result["passCodes"] = [str(i0) for i0 in self.pass_codes]
        elif include_empty:
            result["passCodes"] = []
        if hasattr(self, "previous"):
            result["previous"] = self.previous.to_dict(include_empty=include_empty)
        elif include_empty:
            result["previous"] = SeasonSummary()
        if hasattr(self, "published_at"):
            result["publishedAt"] = str(self.published_at)
        elif include_empty:
            result["publishedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        end: str,
        id_: str,
        name: str,
        namespace: str,
        start: str,
        status: Union[str, StatusEnum],
        pass_codes: Optional[List[str]] = None,
        previous: Optional[SeasonSummary] = None,
        published_at: Optional[str] = None,
    ) -> SeasonSummary:
        instance = cls()
        instance.end = end
        instance.id_ = id_
        instance.name = name
        instance.namespace = namespace
        instance.start = start
        instance.status = status
        if pass_codes is not None:
            instance.pass_codes = pass_codes
        if previous is not None:
            instance.previous = previous
        if published_at is not None:
            instance.published_at = published_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> SeasonSummary:
        instance = cls()
        if not dict_:
            return instance
        if "end" in dict_ and dict_["end"] is not None:
            instance.end = str(dict_["end"])
        elif include_empty:
            instance.end = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "start" in dict_ and dict_["start"] is not None:
            instance.start = str(dict_["start"])
        elif include_empty:
            instance.start = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "passCodes" in dict_ and dict_["passCodes"] is not None:
            instance.pass_codes = [str(i0) for i0 in dict_["passCodes"]]
        elif include_empty:
            instance.pass_codes = []
        if "previous" in dict_ and dict_["previous"] is not None:
            instance.previous = SeasonSummary.create_from_dict(
                dict_["previous"], include_empty=include_empty
            )
        elif include_empty:
            instance.previous = SeasonSummary()
        if "publishedAt" in dict_ and dict_["publishedAt"] is not None:
            instance.published_at = str(dict_["publishedAt"])
        elif include_empty:
            instance.published_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, SeasonSummary]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[SeasonSummary]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[SeasonSummary, List[SeasonSummary], Dict[Any, SeasonSummary]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "end": "end",
            "id": "id_",
            "name": "name",
            "namespace": "namespace",
            "start": "start",
            "status": "status",
            "passCodes": "pass_codes",
            "previous": "previous",
            "publishedAt": "published_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "end": True,
            "id": True,
            "name": True,
            "namespace": True,
            "start": True,
            "status": True,
            "passCodes": False,
            "previous": False,
            "publishedAt": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["DRAFT", "PUBLISHED", "RETIRED"],
        }

    # endregion static methods
